// Methods of use when doing an Ajax application
//
//	function createXMLHttpRequest()
//	function AJAXRequest( method, url, data, callback )
//	function trim (str)
//	function getCookie( name )
//	function setCookie( name, value, expires, path, domain, secure )
//	function days	( n )
//	function hours	( n )
//	function minutes( n )
//	function seconds( n )
//	function removeCookie( name, path, domain )
//	function getCookie(search_name) 
//------------------------------------------------------------
// Make an AJAX request
//
// method:   'GET' or 'POST'
// url:		 URL of servlet
// data:	 Used only for POST. Data to send to the servlet.
// callback: callback method that handles response from servlet. It is
//			 passed the response text that the servlet returns.
//
//
// This example gets a text file from the server and puts it into a div named foo:
//
//	AJAXRequest( 'GET', 'http://localhost:8080/accma/HelloWorld.txt', '',
//				function( text )
//				{	var d = document.getElementById('foo');
//					d.innerHTML=text;
//				}
//	);
//
//  Note that you can't fetch a resource with an AJAX request unless that
//	resource comes from the domain that served the page that contains
//	the request. In the foregoing example, the requesting page must come from the
//  localhost domain, because that's what's requested in the URL.

var msRequestType = "" 	// Holds type of IE ActiveX object to instantiate

function createXMLHttpRequest()
{
	var request;
	
	if (window.XMLHttpRequest)					// Not IE
	{
        request = new XMLHttpRequest();
	}
	else if (window.ActiveXObject)				// Is Some IE Version
	{
        // Instantiate the latest MS ActiveX Objects

		if( msRequestType )		// use previously remembered object type
		{
			request = new ActiveXObject(msRequestType);
		}
		else
		{
			// Figure out which object type to use by trying various possibilities until
			// we succeed. This way we'll get the latest one. See the article at
			// http://blogs.msdn.com/xmlteam/archive/2006/10/23/using-the-right-version-of-msxml-in-internet-explorer.aspx
			// for a discussion of which progIDs to use.

//			var progIDs = [ "Msxml2.XMLHTTP.7.0", "Msxml2.XMLHTTP.6.0", "Msxml2.XMLHTTP.5.0",
//							 "Msxml2.XMLHTTP.4.0", "MSXML2.XMLHTTP.3.0", "MSXML2.XMLHTTP",
//							 "Microsoft.XMLHTTP"];
			
			var progIDs = [ 'Msxml2.DOMDocument.6.0', 'Msxml2.DOMDocument.3.0', 'Microsoft.XMLHTTP'];
			for (var i = 0; i < progIDs.length ; ++i )
			{
				try 
				{
                    request = new ActiveXObject(progIDs[i]);
					if( request )
					{	msRequestType = progIDs[i];
                        break;
                    }
                }
                catch (objException) { /* try next one */ } ;
                
                if( i == progIDs.length )
                	window.alert("Is your browser up to date? (Can't find Microsoft XMLHTTP version 3.0 or 6.0).")
                
            };
        }
    }
    return request;
}

function AJAXRequest( method, url, data, callback )
{
	var request = createXMLHttpRequest();

	// Create an anonymous function to handle state changes as the request is
	// processed. When we get a response from the server, call the user-
	// supplied callback function, passing it the response text.

	request.onreadystatechange = 
		function()
		{	
			if( request.readyState == 4 )	// complete
			{
				if( request.status == 200 )
					callback( request.responseText );
				else
					alert( "AJAX request failed with "+ request.status +"status:\n\t" + method + "\n\t" + url + "\n\t" + request.statusText );
			}
		};

	try
	{	
		var diagnostics  = true;	// turn on diagnostics
		var dontSend 	 = false;	// actually send the request.
		
		if( diagnostics )
			console.debug("Sending AJAX request:\n" + url + ":\n\t" + data );

		if( dontSend )
			callback("{ 'status':'OK', 'token':'testToken', 'message':'server message' } ");
		else
		{
			request.open(method, url, true ); // last argument true for asynchronous processing
	
			// The default behavior is a GET. For a post, change the HTML headers
			// manually before sending the request. Change the content type if it's not
			// plain text. (see: http://www.iana.org/assignments/media-types/)
	
			if (method == "POST") {
				request.setRequestHeader("Connection", "close");
				request.setRequestHeader("Content-Type", "text/plain");
				request.setRequestHeader("Method", "POST " + url + "HTTP/1.1");
			}
	
			request.send( data ? data : "" );
			
			if( diagnostics )
				console.debug("AJAX request sent");
		}
	}
	catch( e )
	{	window.alert( "Error making AJAX Request ("+ method +", "+ url +"):\n" + e );
	}
}

//----------------------------------------------------------------------
// This is the trim12 function from http://blog.stevenlevithan.com/archives/faster-trim-javascript
// For a more, compact, but slightly slower trim, use 
//		return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
//
// If the incoming string is null or '', then '' is returned.

function trim (str) {
	if( !str  || str == '' )
		return '';
		
	var	str = str.replace(/^\s\s*/, ''),
		ws = /\s/,
		i = str.length;
	while (ws.test(str.charAt(--i)));
	return str.slice(0, i + 1);
}

/* Return the cookie's value, or null if the named cookie doesn't exist.
 */
function getCookie( name ) {
	var start = document.cookie.indexOf( name + "=" );
	var len = start + name.length + 1;
	if ( (!start ) && (name != document.cookie.substring( 0, name.length )) )
		return null;

	if( start == -1 )
		return null;
		
	var end = document.cookie.indexOf( ';', len );
	if ( end == -1 )
		end = document.cookie.length;
	
	var value = unescape( document.cookie.substring( len, end ) );
	
    // console.debug("Get Cookie " +name+ "=\"" + (value ? value : "<null>") + "\" (document.cookie: ["+ document.cookie +"])" );	
        
	return value;
}


/*	Set the named cookie to the indicated value.
 *  name: the cookie name.
 *  value: The cookie's value, properly escaped (the value is unescaped by getCookie)
 *	expires: is time from now to expiration, in seconds. To specify days, use days(n) for the
 *			use the date.getGMT() method to get a string suitable for use in this argument.
 *  expires argument.  if empty or missing, then the cookie doesn't expire,
 *  		otherwise, the cookie will expire "expires" milliseconds from now.
 *  		You can use the days(), hours(), minutes(), and seconds() methods to
 *  		easily get the currect number of milliseonds. For example, minutes(10)
 *  		evaluates to the number of milliseconds in 10 minutes.
 *  domain/path: see <http://www.quirksmode.org/js/cookies.html>. Specify the domain and path
 *      to which the cookie will be sent. To make a cookie visible to subdomains, set this
 *      value to the main domain. For example, if the current page is secure.holub.com, then
 *      the cookie is, by default, sent only to that subdomain. To send the cookie to all
 *      subdomains (and the main domain), set its value to "holub.com". Similarly, if a path
 *      is specified, the cookie is sent only to pages that are in the specified directory
 *      (and it's subdirectories). The default is "/".
 */
 
function setCookie( name, value, expires, path, domain, secure )
{
	var expirationTime = new Date();
	expirationTime.setTime( expirationTime.getTime() + (expires ? expires : 0) );

	var cookieValue = name + '=' + escape( value ) +
		( ( expires ) ? ';expires='	+ expirationTime.toGMTString() : '' ) +
		( ( path	) ? ';path=' 	+ path	 : '' ) +
		( ( domain	) ? ';domain=' 	+ domain : '' ) +
		( ( secure	) ? ';secure' 			 : '' );
		
	document.cookie = cookieValue;
		
    // console.debug("Set cookie: " + cookieValue );
}

// Return the number of milliseconds in the indicated interval. For example,
// hours(3) is the number of milliseconds in three hours(). Use this number
// as the "expires" arugment to setCookie().

function days	( n ){ return n * (1000 * 60 * 60 * 24	);}
function hours	( n ){ return n * (1000 * 60 * 60		);}
function minutes( n ){ return n * (1000 * 60			);}
function seconds( n ){ return n * (1000					);}

function removeCookie( name, path, domain )
{
	// Remove doesn't seem to work correctly in Safari (or GWT). Might try to set it to
	// yesterday's date instead of the beginning of time.
	
	var yesterday = new Date();
	yesterday.setTime( yesterday.getTime() - (24*60*60*1000) );
	
	var cookieValue = "cookie (" + name + ") not found";
	if( getCookie( name ) )
	{	var cookieValue = name + '=""' +
			(( path   ) ? ('; path='	+ path	 ): "") +
			(( domain ) ? ('; domain='	+ domain ): "") +
			'; expires=' + yesterday.toGMTString();
			
		document.cookie = cookieValue;	
	}
	
	// console.debug("Removing cookie: " + cookieValue );
}

/*
function getCookie(search_name) 
{
    // note: document.cookie only returns name=value, not the other components
    var tab_cookies = document.cookie.split( ';' );
    for ( i = 0; i < tab_cookies.length; i++ )
    {
        var cookie_tmp = tab_cookies[i].split('=');
        var cookie_name = trim(cookie_tmp[0]);
        
        if (cookie_name==search_name)
        {
        	if (cookie_tmp.length>1) 
        	{
                return unescape( cookie_tmp[1].replace(/^\s+|\s+$/g, '') );
            }
            // cookie is initialized but no value => result = null
            return null;
        }
    }
    return null;
} 
*/